from setuptools import setup, find_packages

setup(name='fqsdfqsdfqsdfqsd',
      version='0.11',
      description='fqsdfqsdfqsdfqsd',
      author='fqsdfqsdfqsdfqsd',
      author_email='fqsdfqsdfqsdfqsdn@gmail.com',
      license='MIT',
      packages=find_packages(),
      install_requires=[
              'python-dateutil',
              'pandas'
          ],
      zip_safe=False)